@@ -19,10 +19,12 @@
 
 package org.jboss.logmanager.config;
 
+import java.util.logging.Formatter;
+
 /**
  * A configuration for a logger formatter.
  *
  * @author <a href="mailto:david.lloyd@redhat.com">David M. Lloyd</a>
  */
-public interface FormatterConfiguration extends NamedConfigurable, ObjectConfigurable, PropertyConfigurable {
+public interface FormatterConfiguration extends NamedConfigurable, ObjectConfigurable<Formatter>, PropertyConfigurable {
 }
