@@ -23,7 +23,6 @@ import java.io.Closeable;
 import java.io.Flushable;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.lang.management.ManagementFactory;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
 import java.text.DateFormatSymbols;
@@ -35,9 +34,13 @@ import java.util.Locale;
 import java.util.logging.ErrorManager;
 import java.util.logging.Formatter;
 import java.util.logging.Level;
+import java.util.regex.Pattern;
+import javax.net.SocketFactory;
+import javax.net.ssl.SSLSocketFactory;
 
 import org.jboss.logmanager.ExtHandler;
 import org.jboss.logmanager.ExtLogRecord;
+import org.wildfly.common.os.Process;
 
 /**
  * A syslog handler for logging to syslogd.
@@ -150,6 +153,7 @@ import org.jboss.logmanager.ExtLogRecord;
  *
  * @author <a href="mailto:jperkins@redhat.com">James R. Perkins</a>
  */
+@SuppressWarnings({"WeakerAccess", "unused"})
 public class SyslogHandler extends ExtHandler {
 
     /**
@@ -294,6 +298,7 @@ public class SyslogHandler extends ExtHandler {
     public static final String DEFAULT_ENCODING = "UTF-8";
     public static final Facility DEFAULT_FACILITY = Facility.USER_LEVEL;
     public static final String NILVALUE_SP = "- ";
+    private static final Pattern PRINTABLE_ASCII_PATTERN = Pattern.compile("[\\P{Print} ]");
 
     static {
         try {
@@ -321,6 +326,7 @@ public class SyslogHandler extends ExtHandler {
     private boolean truncate;
     private int maxLen;
     private boolean blockOnReconnect;
+    private ClientSocketFactory clientSocketFactory;
 
     /**
      * The default class constructor.
@@ -459,9 +465,10 @@ public class SyslogHandler extends ExtHandler {
         this.serverAddress = serverAddress;
         this.port = port;
         this.facility = facility;
-        this.pid = findPid();
+        final long pid = Process.getProcessId();
+        this.pid = (pid != -1 ? Long.toString(pid) : null);
         this.appName = "java";
-        this.hostname = hostname;
+        this.hostname = checkPrintableAscii("host name", hostname);
         this.syslogType = (syslogType == null ? SyslogType.RFC5424 : syslogType);
         if (protocol == null) {
             this.protocol = Protocol.UDP;
@@ -491,10 +498,6 @@ public class SyslogHandler extends ExtHandler {
 
     @Override
     public final void doPublish(final ExtLogRecord record) {
-        // Don't log empty messages
-        if (record.getMessage() == null || record.getMessage().isEmpty()) {
-            return;
-        }
         synchronized (outputLock) {
             init();
             if (out == null) {
@@ -604,7 +607,7 @@ public class SyslogHandler extends ExtHandler {
     }
 
     /**
-     * Gets app name used when formatting the message in RFC5424 format. By default the app name is &quot;java&quot;
+     * Gets app name used when formatting the message in RFC5424 format. By default the app name is &quot;java&quot;.
      *
      * @return the app name being used
      */
@@ -615,7 +618,8 @@ public class SyslogHandler extends ExtHandler {
     }
 
     /**
-     * Sets app name used when formatting the message in RFC5424 format. By default the app name is &quot;java&quot;
+     * Sets app name used when formatting the message in RFC5424 format. By default the app name is &quot;java&quot;. If
+     * set to {@code null} the {@link ExtLogRecord#getProcessName()} will be used.
      *
      * @param appName the app name to use
      *
@@ -625,7 +629,7 @@ public class SyslogHandler extends ExtHandler {
     public void setAppName(final String appName) {
         checkAccess(this);
         synchronized (outputLock) {
-            this.appName = appName;
+            this.appName = checkPrintableAscii("app name", appName);
         }
     }
 
@@ -662,6 +666,19 @@ public class SyslogHandler extends ExtHandler {
         }
     }
 
+    /**
+     * Sets the client socket factory used to create sockets.
+     *
+     * @param clientSocketFactory the client socket factory to use
+     */
+    public void setClientSocketFactory(final ClientSocketFactory clientSocketFactory) {
+        checkAccess(this);
+        synchronized (outputLock) {
+            this.clientSocketFactory = clientSocketFactory;
+            initializeConnection = true;
+        }
+    }
+
     /**
      * Checks whether or not characters below decimal 32, traditional US-ASCII control values expect {@code DEL}, are
      * being escaped or not.
@@ -696,7 +713,7 @@ public class SyslogHandler extends ExtHandler {
     /**
      * Returns the pid being used as the PROCID for RFC5424 messages.
      *
-     * @return the pid
+     * @return the pid or {@code null} if the pid could not be determined
      */
     public String getPid() {
         synchronized (outputLock) {
@@ -861,11 +878,8 @@ public class SyslogHandler extends ExtHandler {
      */
     public void setHostname(final String hostname) {
         checkAccess(this);
-        if (hostname != null && hostname.contains(" ")) {
-            throw new IllegalArgumentException(String.format("Host name '%s' is invalid. Whitespace is now allowed in the host name.", hostname));
-        }
         synchronized (outputLock) {
-            this.hostname = hostname;
+            this.hostname = checkPrintableAscii("host name", hostname);
         }
     }
 
@@ -1006,6 +1020,7 @@ public class SyslogHandler extends ExtHandler {
      * @param out the output stream to write to
      */
     public void setOutputStream(final OutputStream out) {
+        checkAccess(this);
         setOutputStream(out, true);
     }
 
@@ -1037,7 +1052,6 @@ public class SyslogHandler extends ExtHandler {
     }
 
     private void setOutputStream(final OutputStream out, final boolean outputStreamSet) {
-        checkAccess(this);
         OutputStream oldOut = null;
         boolean ok = false;
         try {
@@ -1073,20 +1087,6 @@ public class SyslogHandler extends ExtHandler {
         }
     }
 
-    private static String findPid() {
-        final String name = ManagementFactory.getRuntimeMXBean().getName();
-        String result = name;
-        final int index = name.indexOf("@");
-        if (index > -1) {
-            try {
-                result = Integer.toString(Integer.valueOf(name.substring(0, index)));
-            } catch (NumberFormatException ignore) {
-                // ignore
-            }
-        }
-        return result;
-    }
-
     private void init() {
         if (initializeConnection && !outputStreamSet) {
             if (serverAddress == null || port < 0 || protocol == null) {
@@ -1096,14 +1096,11 @@ public class SyslogHandler extends ExtHandler {
             final OutputStream out;
             // Check the sockets
             try {
-                if (protocol == Protocol.TCP) {
-                    out = new TcpOutputStream(serverAddress, port, blockOnReconnect);
-                } else if (protocol == Protocol.UDP) {
-                    out = new UdpOutputStream(serverAddress, port);
-                } else if (protocol == Protocol.SSL_TCP) {
-                    out = new SslTcpOutputStream(serverAddress, port, blockOnReconnect);
+                final ClientSocketFactory clientSocketFactory = getClientSocketFactory();
+                if (protocol == Protocol.UDP) {
+                    out = new UdpOutputStream(clientSocketFactory);
                 } else {
-                    throw new IllegalStateException("Invalid protocol: " + protocol);
+                    out = new TcpOutputStream(clientSocketFactory, blockOnReconnect);
                 }
                 setOutputStream(out, false);
             } catch (IOException e) {
@@ -1190,24 +1187,35 @@ public class SyslogHandler extends ExtHandler {
             buffer.append(' ');
         }
         // Set the host name
-        if (hostname == null) {
-            buffer.append(NILVALUE_SP);
+        final String recordHostName = record.getHostName();
+        if (hostname != null) {
+            buffer.appendPrintUSASCII(hostname, 255).append(' ');
+        } else if (recordHostName != null) {
+            buffer.appendPrintUSASCII(recordHostName, 255).append(' ');
         } else {
-            buffer.appendUSASCII(hostname, 255).append(' ');
+            buffer.append(NILVALUE_SP);
         }
         // Set the app name
-        if (appName == null) {
-            buffer.appendUSASCII(NILVALUE_SP);
-        } else {
-            buffer.appendUSASCII(appName, 48);
+        final String recordProcName = record.getProcessName();
+        if (appName != null) {
+            buffer.appendPrintUSASCII(appName, 48);
             buffer.append(' ');
+        } else if (recordProcName != null) {
+            buffer.appendPrintUSASCII(recordProcName, 48);
+            buffer.append(' ');
+        } else {
+            buffer.appendUSASCII(NILVALUE_SP);
         }
         // Set the procid
-        if (pid == null) {
-            buffer.appendUSASCII(NILVALUE_SP);
-        } else {
-            buffer.appendUSASCII(pid, 128);
+        final long recordProcId = record.getProcessId();
+        if (recordProcId != -1) {
+            buffer.append(recordProcId);
             buffer.append(' ');
+        } else if (pid != null) {
+            buffer.appendPrintUSASCII(pid, 128);
+            buffer.append(' ');
+        } else {
+            buffer.appendUSASCII(NILVALUE_SP);
         }
         // Set the msgid
         final String msgid = record.getLoggerName();
@@ -1217,7 +1225,7 @@ public class SyslogHandler extends ExtHandler {
             buffer.appendUSASCII("root-logger");
             buffer.append(' ');
         } else {
-            buffer.appendUSASCII(msgid, 32);
+            buffer.appendPrintUSASCII(msgid, 32);
             buffer.append(' ');
         }
         // Set the structured data
@@ -1265,20 +1273,53 @@ public class SyslogHandler extends ExtHandler {
         buffer.append(' ');
 
         // Set the host name
-        if (hostname == null) {
-            // TODO might not be the best solution
-            buffer.appendUSASCII("UNKNOWN_HOSTNAME").append(' ');
-        } else {
+        final String recordHostName = record.getHostName();
+        if (hostname != null) {
             buffer.appendUSASCII(hostname).append(' ');
+        } else if (recordHostName != null) {
+            buffer.appendUSASCII(recordHostName).append(' ');
+        } else {
+            buffer.appendUSASCII("UNKNOWN_HOSTNAME").append(' ');
         }
         // Set the app name and the proc id
-        if (appName != null && pid != null) {
-            buffer.appendUSASCII(appName).append('[').appendUSASCII(pid).append(']').appendUSASCII(": ");
-        } else if (appName != null) {
-            buffer.append(appName).append(": ");
+        final String recordProcName = record.getProcessName();
+        boolean colon = false;
+        if (appName != null) {
+            buffer.appendUSASCII(appName);
+            colon = true;
+        } else if (recordProcName != null) {
+            buffer.appendUSASCII(recordProcName);
+            colon = true;
+        }
+        final long recordProcId = record.getProcessId();
+        if (recordProcId != -1) {
+            buffer.append('[').append(recordProcId).append(']');
+            colon = true;
         } else if (pid != null) {
-            buffer.append('[').appendUSASCII(pid).append(']').append(": ");
+            buffer.append('[').appendUSASCII(pid).append(']');
+            colon = true;
+        }
+        if (colon) {
+            buffer.append(':').append(' ');
         }
         return buffer.toArray();
     }
+
+    private ClientSocketFactory getClientSocketFactory() {
+        synchronized (outputLock) {
+            if (clientSocketFactory != null) {
+                return clientSocketFactory;
+            }
+            final SocketFactory socketFactory = (protocol == Protocol.SSL_TCP ? SSLSocketFactory.getDefault() : SocketFactory.getDefault());
+            return ClientSocketFactory.of(socketFactory, serverAddress, port);
+        }
+    }
+
+    private static String checkPrintableAscii(final String name, final String value) {
+        if (value != null && PRINTABLE_ASCII_PATTERN.matcher(value).find()) {
+            final String upper = Character.toUpperCase(name.charAt(0)) + name.substring(1);
+            throw new IllegalArgumentException(String.format("%s '%s' is invalid. The %s must be printable ASCII characters with no spaces.", upper, value, name));
+        }
+        return value;
+    }
 }
