@@ -436,6 +436,15 @@ public final class FilterTests {
         assertFalse("Handler was run", ran.get());
     }
 
+    @Test
+    public void regexFilterExceptionNullMessageTest(){
+        final ExtLogRecord logRecord = new ExtLogRecord(Level.ALL, null, null);
+        final Filter filter = new RegexFilter("test");
+        boolean isLoggable = filter.isLoggable(logRecord);
+        assertFalse(isLoggable);
+        assertNull(logRecord.getFormattedMessage());
+    }
+
     @Test
     public void testSubstitueFilter0() {
         final Filter filter = new SubstituteFilter(Pattern.compile("test"), "lunch", true);
@@ -490,6 +499,14 @@ public final class FilterTests {
         assertEquals("Substitution was not correctly applied", "This is a lunches lunches", record.getFormattedMessage());
     }
 
+    @Test
+    public void substituteFilterExceptionNullMessageTest(){
+        final ExtLogRecord logRecord = new ExtLogRecord(Level.ALL, null, null);
+        final Filter filter = new SubstituteFilter(Pattern.compile("test"), "lunch", true);
+        filter.isLoggable(logRecord);
+        assertEquals("null", logRecord.getFormattedMessage());
+    }
+
 
 
     private static final class MessageCheckingHandler extends Handler {
